import { world, system } from "@minecraft/server";
import { ModalFormData, ActionFormData } from "@minecraft/server-ui";

// Define the BanDataBase class with an expiration time
class BanDataBase {
    BannedUser = "";
    BanReason = "";
    ExpirationTime = 0; // Time in milliseconds when the ban expires (0 for permanent)

    constructor(banDataBase) {
        if (!banDataBase) return this;
        Object.assign(this, banDataBase);
    }
}

class Ban_Systems {

    Ban_Form(player) {
        const OnlinePlayers = ['None'].concat(...world.getAllPlayers().map(p => p.name));
        const form = new ModalFormData()
            .title('Ban Player')
            .dropdown('Select Player', OnlinePlayers, 0)
            .textField('Ban Reason', 'Enter ban reason')
            .slider('Ban Duration (Days)', 0, 30, 1) // Days
            .slider('Ban Duration (Hours)', 0, 23, 1) // Hours
            .slider('Ban Duration (Minutes)', 0, 59, 1); // Minutes

        form.show(player).then((data) => {
            if (data.canceled) return;

            const target = this.GetPlayerWithName(OnlinePlayers[data.formValues[0]]);
            if (!target) return player.sendMessage("§cPlayer is not online!");

            const banDuration = 
                (data.formValues[2] * 24 * 60 * 60 * 1000) +  // Days to milliseconds
                (data.formValues[3] * 60 * 60 * 1000) +       // Hours to milliseconds
                (data.formValues[4] * 60 * 1000);             // Minutes to milliseconds

            const expirationTime = Date.now() + banDuration;  // Current time + ban duration

            // Add ban data with expiration time
            const banData = JSON.parse(world.getDynamicProperty('BanData') || "[]");
            banData.push(new BanDataBase({
                BannedUser: target.name,
                BanReason: data.formValues[1],
                ExpirationTime: expirationTime
            }));
            world.setDynamicProperty('BanData', JSON.stringify(banData));

            // Kick the target player with a reason
            target.runCommandAsync(`kick "${target.name}" §l§cYOU ARE BANNED\n§l§cReason:§r ${data.formValues[1]}\n\nRequest an unban from an admin`);
        });
    }

    UnbanForm(player) {
        let banData = JSON.parse(world.getDynamicProperty('BanData') || "[]");
        const form = new ActionFormData().title('UnBan Player');
        
        banData.forEach((ban, index) => {
            form.button(ban.BannedUser, "textures/ui/icon_steve");
        });
        form.button("Exit");

        form.show(player).then((data) => {
            if (data.selection < banData.length) {
                // Remove the unbanned player from the ban list
                banData.splice(data.selection, 1);  
                world.setDynamicProperty('BanData', JSON.stringify(banData));
            }
        });
    }

    BanType(player) {
        const form = new ActionFormData()
            .button("Ban")
            .button("UnBan");
        form.show(player).then((data) => {
            if (data.selection === 0) return this.Ban_Form(player);
            if (data.selection === 1) return this.UnbanForm(player);
        });
    }

    GetPlayerWithName(PlayerName) {
        return world.getAllPlayers().find(player => player.name === PlayerName);
    }

    // Helper function to calculate remaining time
    getRemainingTime(expirationTime) {
        const currentTime = Date.now();
        const timeLeft = expirationTime - currentTime;

        const days = Math.floor(timeLeft / (1000 * 60 * 60 * 24));
        const hours = Math.floor((timeLeft % (1000 * 60 * 60 * 24)) / (1000 * 60 * 60));
        const minutes = Math.floor((timeLeft % (1000 * 60 * 60)) / (1000 * 60));

        return { days, hours, minutes };
    }
}

export const Ban_System = new Ban_Systems();

// Kicking banned players upon login
world.afterEvents.playerSpawn.subscribe((data) => {
    const player = data.player;
    const banData = JSON.parse(world.getDynamicProperty('BanData') || "[]");
    
    const bannedPlayer = banData.find(ban => ban.BannedUser === player.name);
    if (bannedPlayer) {
        if (bannedPlayer.ExpirationTime > Date.now()) {
            const remainingTime = Ban_System.getRemainingTime(bannedPlayer.ExpirationTime);
            player.runCommandAsync(`kick "${player.name}" §l§cYOU ARE BANNED!\n§l§cReason:§r ${bannedPlayer.BanReason}\n§cDays: ${remainingTime.days}\n§cHours: ${remainingTime.hours}\n§cMinutes: ${remainingTime.minutes}\n\nRequest an unban from an admin`);
        } else {
            // Remove expired ban
            const updatedBanData = banData.filter(ban => ban.BannedUser !== player.name);
            world.setDynamicProperty('BanData', JSON.stringify(updatedBanData));
        }
    }
});

// Periodically check for expired bans and remove them
system.runInterval(() => {
    let banData = JSON.parse(world.getDynamicProperty('BanData') || "[]");
    const currentTime = Date.now();
    const updatedBanData = banData.filter(ban => ban.ExpirationTime > currentTime);

    // Update dynamic property if any bans expired
    if (updatedBanData.length !== banData.length) {
        world.setDynamicProperty('BanData', JSON.stringify(updatedBanData));
    }

    for (const ban of updatedBanData) {
        for (const player of world.getAllPlayers()) {
            if (player.name === ban.BannedUser) {
                const remainingTime = Ban_System.getRemainingTime(ban.ExpirationTime);
                player.runCommandAsync(`kick "${ban.BannedUser}" \n§l§cYOU ARE BANNED!\n§l§cReason:§r ${ban.BanReason}\n§c§lDays: ${remainingTime.days}\n§cHours: ${remainingTime.hours}\n§cMinutes: ${remainingTime.minutes}`);
            }
        }
    }
}); 

