import { ModalFormData } from "@minecraft/server-ui";
import { world } from "@minecraft/server";

// Configuration object for enabling/disabling features
const FeatureConfig = {
    Announcements: world.getDynamicProperty("Announcements") ?? true,
    Warps: world.getDynamicProperty("Warps") ?? true,
    TPA: world.getDynamicProperty("TPA") ?? true,
    Settings: world.getDynamicProperty("Settings") ?? true,
    Reports: world.getDynamicProperty("Reports") ?? true,
    AdminPanel: world.getDynamicProperty("AdminPanel") ?? true,
};

// Function to toggle features and save settings
export function ToggleFeature(player) {
    const form = new ModalFormData()
        .title('Toggle Features')
        .toggle('Announcements', FeatureConfig.Announcements)
        .toggle('Warps', FeatureConfig.Warps)
        .toggle('TPA', FeatureConfig.TPA)
        .toggle('Settings', FeatureConfig.Settings)
        .toggle('Reports', FeatureConfig.Reports)
        .toggle('Admin Panel', FeatureConfig.AdminPanel);

    form.show(player).then((data) => {
        if (data.canceled) return;

        // Update the FeatureConfig based on form input
        FeatureConfig.Announcements = data.formValues[0];
        FeatureConfig.Warps = data.formValues[1];
        FeatureConfig.TPA = data.formValues[2];
        FeatureConfig.Settings = data.formValues[3];
        FeatureConfig.Reports = data.formValues[4];
        FeatureConfig.AdminPanel = data.formValues[5];

        // Save updated settings to the world
        world.setDynamicProperty("Announcements", FeatureConfig.Announcements);
        world.setDynamicProperty("Warps", FeatureConfig.Warps);
        world.setDynamicProperty("TPA", FeatureConfig.TPA);
        world.setDynamicProperty("Settings", FeatureConfig.Settings);
        world.setDynamicProperty("Reports", FeatureConfig.Reports);
        world.setDynamicProperty("AdminPanel", FeatureConfig.AdminPanel);

        player.sendMessage('§aFeature settings updated.');
    });
}

// Export the configuration for use in other files
export { FeatureConfig };
