
import { Player, world } from "@minecraft/server"
import { ActionFormData, ModalFormData } from "@minecraft/server-ui"

class Warp_Options {
    location = {}
    displayName = ""
    /**
     * @param {Warp_Options} Warp_Option 
     */
    constructor(Warp_Option) {
        if (!Warp_Option) return
        Object.assign(this, Warp_Option)
    }
}

class Warp_Functions {

    /**
     * @description This function will return the player's objective 
     * @param {Player} player 
     */
    GetObjective(player) {
        if (!world.scoreboard.getObjective(`${player.name}`))
            world.scoreboard.addObjective(`${player.name}`, `${player.name}'s Warps`)
        return world.scoreboard.getObjective(`${player.name}`)
    }

    /**
     * @description This function will get the warps from the player
     * @param {Player} player 
     * @returns {Warp_Options[]}
     */
    GetWarps(player) {
        let Warps = []
        for (let participant of this.GetObjective(player).getParticipants()) {
            let data = new Warp_Options(JSON.parse(participant.displayName))
            Warps.push(data)
        }
        return Warps
    }

    /**
     * 
     * @param {Player} player 
     */
    CreateWarp(player) {
        const form = new ModalFormData()
            .title(`§l§8Create Warp`)
            .textField(`Name`, `Ex: My Warp`)
        form.show(player).then((data) => {
            if (!data)
                return
            this.GetObjective(player).addScore(JSON.stringify(new Warp_Options({ location: player.location, displayName: data.formValues[0] })), 0)
            this.MainMenu(player)
        })
    }

    /**
     * @param {Player} player 
     */
    MainMenu(player) {
        let Warps = this.GetWarps(player)

        const form = new ActionFormData()
            .title(`§l§8Your Warps`)
        for (let i = 0; i < Warps.length; i++) {
            let Warp = Warps[i]
            form.button(`${Warp.displayName}\n§l§f[§d WARP §f]`)
        }
        form.button(`§bCreate`)
        form.button(`§4Exit`)
        form.show(player).then((data) => {
            if (data.selection < Warps.length) {
                return this.WarpMenu(player, Warps, data.selection)
            }
            if (data.selection === Warps.length)
                return this.CreateWarp(player)
        })
    }
    /**
     * 
     * @param {Player} player 
     * @param {Warp_Options[]} Warps 
     * @param {number} index 
     */
    WarpMenu(player, Warps, index) {
        let Warp = Warps[index]
        const form = new ActionFormData()
            .title(Warp.displayName)
            .button(`§bTeleport`)
            .button(`§4Delete`)
        form.show(player).then(data => {
            if (data.selection === 0)
                return player.teleport(Warp.location)
            else
                if (data.selection === 1) {
                    this.GetObjective(player).removeParticipant(JSON.stringify(Warp))
                    this.MainMenu(player)
                }
        })
    }
}

export const Warp_Function = new Warp_Functions()