import { system, world } from '@minecraft/server';
import { ModalFormData } from '@minecraft/server-ui'

class TPAs {
    constructor() {
        this.requestCooldown = new Map(); // Track cooldowns for players
        this.cooldownTime = 60; // Cooldown time in seconds
    }

    TPA_menu(player) {
        const OnlinePlayers = ['None'].concat(...world.getAllPlayers().filter(e => e.name.toLowerCase() !== player.name.toLowerCase()).map(player => player.name));
        const form = new ModalFormData()
            .dropdown('Who Do You Want To TPA to?', OnlinePlayers);
        form.show(player).then((data) => {
            const target = this.GetPlayerWithName(OnlinePlayers[data.formValues[0]]);
            if (target) {
                if (target.getDynamicProperty("TPAREQUEST") == null || target.getDynamicProperty("TPAREQUEST") == false) {
                    this.TPA_Request(target, player);
                } else {
                    player.sendMessage(`§c${target.name} has their TPA Requests Off!`);
                }
            }
        });
    }

    TPA_Request(target, player) {
        if (this.isOnCooldown(player, target)) {
            player.sendMessage(`§cYou have already sent a TPA request to ${target.name}. Please wait before sending another.`);
            return;
        }

        target.sendMessage(`§a${player.name} wants to teleport to you. Type "!accept" to accept or "!deny" to deny the request.`);
        player.sendMessage(`§aYou have requested to teleport to ${target.name}.`);

        // Temporarily store the request details
        system.runTimeout(() => {
            target.setDynamicProperty("tpaRequest", player.name);
        }, 0);

        // Add the player to the cooldown tracker
        this.addToCooldown(player, target);
    }

    handleResponse(sender, message) {
        const requestorName = sender.getDynamicProperty("tpaRequest");
        if (!requestorName) return;

        const requestor = this.GetPlayerWithName(requestorName);
        if (!requestor) {
            sender.sendMessage(`§cThe player who sent the TPA request is no longer online.`);
            sender.setDynamicProperty("tpaRequest", null);
            return;
        }

        if (message.toLowerCase() === "!accept") {
            system.run(() => {
                requestor.teleport({ x: sender.location.x, y: sender.location.y, z: sender.location.z });
            })

            requestor.sendMessage(`§aYou have teleported to ${sender.name}`);
            sender.sendMessage(`§aYou have accepted the TPA request from ${requestor.name}`);
        } else if (message.toLowerCase() === "!deny") {
            requestor.sendMessage(`§c${sender.name} has denied your TPA request.`);
            sender.sendMessage(`§cYou have denied the TPA request from ${requestor.name}`);
        }

        // Clear the request
        sender.setDynamicProperty("tpaRequest", null);
    }

    GetPlayerWithName(PlayerName) {
        PlayerName = PlayerName.toLowerCase();
        for (let player of world.getAllPlayers()) {
            if (player.name.toLowerCase() === PlayerName) {
                return player;
            }
        }
        return undefined;
    }

    handleTPACommand(sender, targetName) {
        const target = this.GetPlayerWithName(targetName);
        if (!target) {
            sender.sendMessage(`§cPlayer ${targetName} is not online.`);
            return;
        }

        if (target.getDynamicProperty("TPAREQUEST") == null || target.getDynamicProperty("TPAREQUEST") == false) {
            this.TPA_Request(target, sender);
        } else {
            sender.sendMessage(`§c${target.name} has their TPA requests off!`);
        }
    }

    isOnCooldown(player, target) {
        const playerName = player.name;
        const targetName = target.name;

        if (!this.requestCooldown.has(playerName)) {
            this.requestCooldown.set(playerName, new Map());
        }

        const playerCooldowns = this.requestCooldown.get(playerName);
        const lastRequestTime = playerCooldowns.get(targetName);

        if (lastRequestTime) {
            const timeElapsed = (Date.now() - lastRequestTime) / 1000; // Convert to seconds
            if (timeElapsed < this.cooldownTime) {
                return true;
            }
        }

        return false;
    }

    addToCooldown(player, target) {
        const playerName = player.name;
        const targetName = target.name;

        if (!this.requestCooldown.has(playerName)) {
            this.requestCooldown.set(playerName, new Map());
        }

        const playerCooldowns = this.requestCooldown.get(playerName);
        playerCooldowns.set(targetName, Date.now());
    }
}

export const TPA = new TPAs();

// Subscribe to the chat event for the "!tpa [username]" command
world.beforeEvents.chatSend.subscribe(event => {
    const message = event.message;
    const sender = event.sender;

    if (message.toLowerCase().startsWith('!tpa ')) {
        const targetName = message.substring(5).trim();
        let sanitizedName = targetName.startsWith('@') ? targetName.substring(1) : targetName;
        TPA.handleTPACommand(sender, sanitizedName);
        event.cancel = true;
    } else if (message.toLowerCase() === "!accept" || message.toLowerCase() === "!deny") {
        TPA.handleResponse(sender, message);
        event.cancel = true;
    }
});
